package com.rjconsultores.ventaboletos.relatorios.impl;

import java.math.BigDecimal;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

import com.rjconsultores.ventaboletos.relatorios.utilitarios.DataSource;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.Relatorio;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.RelatorioDepositoBean;
import com.rjconsultores.ventaboletos.utilerias.DateUtil;

import net.sf.jasperreports.engine.data.JRBeanCollectionDataSource;

public class RelatorioDepositos extends Relatorio {
	
	private static Logger log = Logger.getLogger(RelatorioDepositos.class);

	private List<RelatorioDepositoBean> lsDadosRelatorio;

	public RelatorioDepositos(Map<String, Object> parametros, Connection conexao, final String... nomeSubReporte) {
		super(parametros, conexao);

		try {
			this.setCustomDataSource(new DataSource(this) {
				@Override
				public void initDados() throws Exception {

					Connection conexao = this.relatorio.getConexao();

					Map<String, Object> parametros = this.relatorio.getParametros();

					Integer puntoventaId = parametros.get("puntoventaId") != null ? Integer.parseInt(parametros.get("puntoventaId").toString()) : null;
					Integer empresaId = parametros.get("empresaId") != null ? Integer.parseInt(parametros.get("empresaId").toString()) : null;
					Boolean filtrarPendentes = parametros.get("filtrarPendentes") != null ? Boolean.parseBoolean(parametros.get("filtrarPendentes").toString()) : false;
					String fecInicio = parametros.get("fecInicio").toString() + " 00:00:00";
					String fecFinal = parametros.get("fecFinal").toString() + " 23:59:59";

					String sql = getSQL(puntoventaId, empresaId, filtrarPendentes);

					PreparedStatement stmt = conexao.prepareStatement(sql);
					
					ResultSet rs = null;
					SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy hh:mm:ss");

					stmt.setTimestamp(1, new java.sql.Timestamp(sdf.parse(fecInicio).getTime()));
					stmt.setTimestamp(2, new java.sql.Timestamp(sdf.parse(fecFinal).getTime()));
					
					rs = stmt.executeQuery();
					rs.setFetchSize(500);

					lsDadosRelatorio = new ArrayList<RelatorioDepositoBean>();
					
					Integer puntoventaIdAux = null;
					Integer empresaIdAux = null;
					BigDecimal pendente = BigDecimal.ZERO;
					BigDecimal totalFechamento = BigDecimal.ZERO;
					Date dtMotivo = null;
					Long fechamentocntcorrenteId = null;

					while (rs.next()) {
						RelatorioDepositoBean deposito = new RelatorioDepositoBean();
						deposito.setNombempresa(rs.getString("nombempresa"));
						deposito.setNombpuntoventa(rs.getString("nombpuntoventa"));
						deposito.setNumpuntoventa(rs.getString("numpuntoventa"));
						deposito.setDtmotivo(rs.getDate("dtmotivo"));
						deposito.setSaldo(rs.getBigDecimal("saldo").multiply(BigDecimal.valueOf(-1)));
						deposito.setVrdeposito(rs.getBigDecimal("vlrBoletoDeposito"));	
						deposito.setVrfechamento(rs.getBigDecimal("vrfechamento"));
						deposito.setEmpresaId(rs.getInt("empresaId"));
						deposito.setPuntoventaId(rs.getInt("puntoventaId"));
						
						
						if (! filtrarPendentes){
							deposito.setNumdeposito(rs.getString("numdeposito"));
						}
												
						boolean isPontoVendaEmpresaDiferente = (puntoventaIdAux == null || !puntoventaIdAux.equals(deposito.getPuntoventaId())) ||
								   							   (empresaIdAux == null || !empresaIdAux.equals(deposito.getEmpresaId()));
						
						if(fechamentocntcorrenteId == null) {
							fechamentocntcorrenteId = rs.getLong("fechamentocntcorrenteId");
						} if(!isPontoVendaEmpresaDiferente && dtMotivo != null && DateUtil.compareOnlyDate(dtMotivo, deposito.getDtmotivo()) == 0 && !fechamentocntcorrenteId.equals(rs.getLong("fechamentocntcorrenteId"))) {
							pendente = deposito.getVrfechamento();
							totalFechamento = totalFechamento.add(deposito.getVrfechamento());
							fechamentocntcorrenteId = rs.getLong("fechamentocntcorrenteId");
						}
						
						if(isPontoVendaEmpresaDiferente || (dtMotivo == null || DateUtil.compareOnlyDate(dtMotivo, deposito.getDtmotivo()) != 0)) {
							
							if(isPontoVendaEmpresaDiferente) {
								puntoventaIdAux = deposito.getPuntoventaId();
								empresaIdAux = deposito.getEmpresaId();
								pendente = deposito.getVrfechamento();
								dtMotivo = deposito.getDtmotivo();
								totalFechamento = totalFechamento.add(deposito.getVrfechamento());
								fechamentocntcorrenteId = rs.getLong("fechamentocntcorrenteId");
							}

							if(dtMotivo == null || DateUtil.compareOnlyDate(dtMotivo, deposito.getDtmotivo()) != 0) {
								pendente = deposito.getVrfechamento();
								totalFechamento = totalFechamento.add(deposito.getVrfechamento());
								fechamentocntcorrenteId = rs.getLong("fechamentocntcorrenteId");
							}
							dtMotivo = deposito.getDtmotivo();
						}
						
						pendente = pendente.subtract(deposito.getVrdeposito());
						deposito.setVrpendente(pendente);
						
						lsDadosRelatorio.add(deposito);
					}

					if (lsDadosRelatorio.size() > 0) {
						setLsDadosRelatorio(lsDadosRelatorio);
					}
					parametros.put("totalFechamento", totalFechamento);
					setNomeSubReporte(nomeSubReporte);
				}

			});
		} catch (Exception e) {
			log.error("", e);
		}
	}

	public void setLsDadosRelatorio(List<RelatorioDepositoBean> lsDadosRelatorio) {
		this.setCollectionDataSource(new JRBeanCollectionDataSource(lsDadosRelatorio));
		this.lsDadosRelatorio = lsDadosRelatorio;
	}

	@Override
	protected void processaParametros() throws Exception {
	}

	private String getSQL(Integer puntoVentaId, Integer empresaId, boolean filtrarPendentes) {

		StringBuilder sql = new StringBuilder();
		
		sql.append(" SELECT DISTINCT tmp.nombempresa, ");
		sql.append("   tmp.nombpuntoventa, ");
		sql.append("   tmp.numpuntoventa, ");
		sql.append("   tmp.dtmotivo, ");
		sql.append("   tmp.saldo, ");
		sql.append("   tmp.vrfechamento, ");
		sql.append("   tmp.fechamentocntcorrenteId, ");
		
		if (! filtrarPendentes){
			sql.append("   tmp.numdeposito, ");
		} 

		sql.append("   tmp.vlrBoletoDeposito, ");	
		sql.append("   tmp.puntoventaId, ");
		sql.append("   tmp.empresaId ");
		sql.append(" FROM ");
		sql.append("   ( SELECT DISTINCT e.nombempresa, ");
		sql.append("     p.nombpuntoventa, ");
		sql.append("     p.numpuntoventa, ");
		sql.append("     p.puntoventa_id as puntoventaId, ");
		sql.append("     e.empresa_id as empresaId, ");
		sql.append("     f.fecfechamento                         AS dtmotivo, ");
		sql.append("     ( SUM(COALESCE(fd.valor_pago,fbol.valordocumento,0)) - f.total ) AS saldo, ");
		sql.append("     SUM(COALESCE(fd.valor_pago,fbol.valordocumento,0))    AS vlrBoletoDeposito, ");
		sql.append("     f.total                                 AS vrfechamento, ");
		
		if (! filtrarPendentes){
			sql.append("     fdp.NUMDEPOSITO as numdeposito, ");
		}

		sql.append("     f.FECHAMENTOCNTCORRENTE_ID as fechamentocntcorrenteId ");
		sql.append("   FROM fechamento_cntcorrente f ");
		sql.append("   INNER JOIN punto_venta p ");
		sql.append("      ON p.puntoventa_id = f.puntoventa_id ");
		sql.append("   INNER JOIN empresa e ");
		sql.append("      ON e.empresa_id = f.empresa_id ");
		sql.append("   INNER JOIN fechamento_paramptovta fppv "); 
		sql.append("      ON p.puntoventa_id = fppv.puntoventa_id ");
		sql.append("      AND e.empresa_id = fppv.empresa_id ");
		sql.append("      AND tipopagamento = 'D' ");
		sql.append("      AND fppv.activo = 1 ");
		sql.append("   LEFT JOIN fechamento_cct_deposito fd ");
		sql.append("      ON fd.fechamentocntcorrente_id = f.fechamentocntcorrente_id ");
		sql.append("      AND fd.activo = 1 ");
		sql.append("   LEFT JOIN FECHAMENTO_DEPOSITO fdp ");
		sql.append("      ON fdp.FECHAMENTODEPOSITO_ID = fd.FECHAMENTODEPOSITO_ID ");
		sql.append("      AND fdp.activo = 1 ");
		sql.append("   LEFT JOIN empresa_contabancaria ecb ");
		sql.append("      ON ecb.EMPRESACONTABANCARIA_ID = fdp.EMPRESACONTABANCARIA_ID ");
		sql.append("      AND ecb.activo = 1 ");
		sql.append("   LEFT JOIN fechamento_boleto fbol ON f.fechamentocntcorrente_id = fbol.fechamentocntcorrente_id and fbol.indboletoquitado=1 ");
		
		sql.append("   WHERE f.fecfechamento BETWEEN ? AND ? ");
		sql.append("     AND f.activo = 1 ");
		sql.append("     AND e.activo = 1 ");
		sql.append("     AND p.activo = 1 ");
		
		if (puntoVentaId != null && puntoVentaId != -1){
			sql.append(" and f.puntoventa_id = " + puntoVentaId);
		}
		if (empresaId != null && empresaId != -1){
			sql.append(" and e.empresa_id = " + empresaId);
		}
		sql.append(" GROUP BY e.nombempresa,  ");
		sql.append("   p.nombpuntoventa,  ");
		sql.append("   p.numpuntoventa,  ");
		sql.append("   p.puntoventa_id, ");
		sql.append("   e.empresa_id, ");
		sql.append("   f.fecfechamento,  ");
		sql.append("   f.FECHAMENTOCNTCORRENTE_ID,  ");
		
		if (!filtrarPendentes){
			sql.append("   fdp.NUMDEPOSITO, ");
		}
	
		sql.append("   f.total ) tmp ");
		
		if (filtrarPendentes){
			sql.append(" where ");
			sql.append(" tmp.saldo < tmp.vlrBoletoDeposito and tmp.saldo <> 0");
		}
		sql.append(" ORDER BY tmp.nombempresa,  ");
		sql.append("   tmp.nombpuntoventa,  ");
		sql.append("   tmp.dtmotivo, ");
		sql.append("   tmp.vlrBoletoDeposito,  ");
		sql.append("   tmp.puntoventaId, ");
		sql.append("   tmp.empresaId ");		
		return sql.toString();
	}
}
