package com.rjconsultores.ventaboletos.web.gui.controladores.comissao;

import java.text.DecimalFormat;
import java.util.Locale;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.Executions;
import org.zkoss.zk.ui.WrongValueException;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zul.Messagebox;

import com.rjconsultores.ventaboletos.entidad.ItemDesconto;
import com.rjconsultores.ventaboletos.service.ItemDescontoService;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.MyTextbox;
import com.rjconsultores.ventaboletos.web.utilerias.MyTextboxDecimal;

@Scope("prototype")
@Controller("editarItemDescontoController")
public class EditarItemDescontoController extends MyGenericForwardComposer {
	
	private static final long serialVersionUID = 1L;
	
	private static final Logger log = Logger.getLogger(EditarItemDescontoController.class);

	private static final Locale LOC = new Locale("pt","br");
	
	@Autowired
	private ItemDescontoService itemDescontoService;

	private ItemDesconto itemDesconto;
	private MyTextbox txtNomitemdesconto;
	private MyTextboxDecimal txtPreco;
	private MyListbox itemDescontoList;
	
    @Override
    public void doAfterCompose(Component comp) throws Exception {
        super.doAfterCompose(comp);
        
        itemDesconto = (ItemDesconto) Executions.getCurrent().getArg().get("itemDesconto");
		itemDescontoList = (MyListbox) Executions.getCurrent().getArg().get("itemDescontoList");

		if (itemDesconto.getItemdescontoId() != null) {
			txtNomitemdesconto.setValue(itemDesconto.getNomitemdesconto());
			txtPreco.setValue(DecimalFormat.getInstance(LOC).format(itemDesconto.getPreco()));
		}

        txtNomitemdesconto.focus();
    }
    
    public void onClick$btnSalvar(Event ev) throws InterruptedException {
		try {
			txtNomitemdesconto.getValue();
			txtPreco.getValue();
			
			itemDesconto.setNomitemdesconto(txtNomitemdesconto.getValue());
			itemDesconto.setPreco(txtPreco.getValueDecimal());

			if (itemDesconto.getItemdescontoId() == null) {
				itemDescontoService.suscribir(itemDesconto);
				itemDescontoList.addItemNovo(itemDesconto);
			} else {
				itemDescontoService.actualizacion(itemDesconto);
			}
			
			itemDescontoList.updateItem(itemDesconto);

			closeWindow();

		} catch (WrongValueException ex) {
			throw (WrongValueException) ex;
		} catch (Exception ex) {
			log.error(ex);
			Messagebox.show(Labels.getLabel("MSG.Error").concat(" - ").concat(ex.toString()),
					Labels.getLabel("editarItemDescontoController.window.title"),
					Messagebox.OK, Messagebox.ERROR);
		}    	
    }
    
    public void onClick$btnApagar(Event ev) {
		try {
			int resp = Messagebox.show(
					Labels.getLabel("editarItemDescontoController.MSG.borrarPergunta"),
					Labels.getLabel("editarItemDescontoController.window.title"),
					Messagebox.YES | Messagebox.NO, Messagebox.QUESTION);

			if (resp == Messagebox.YES) {

				itemDescontoService.borrar(itemDesconto);

				log.warn("Usurio " + UsuarioLogado.getUsuarioLogado().getNombusuario() + " excluiu o item desconto " + itemDesconto.getItemdescontoId());

				Messagebox.show(
						Labels.getLabel("editarItemDescontoController.MSG.borrarOK"),
						Labels.getLabel("editarItemDescontoController.window.title"),
						Messagebox.OK, Messagebox.INFORMATION);

				itemDescontoList.removeItem(itemDesconto);

				closeWindow();
			}
		} catch (Exception ex) {
			log.error(ex);
		}
	}

}
